"use strict";
/**************************************************************************************************

	This software module is released under the Modified BSD License.

	Mersenne Twister

		Copyright © 1997 - 2002, Makoto Matsumoto and Takuji Nishimura. All rights reserved.

		Contributors:
		- Sean McCullough
		- Eurythrace Perseides

		A C-program for MT19937, with initialization improved 2002-Jan-26. Coded by Takuji
		Nishimura and Makoto Matsumoto (m-mat@math.sci.hiroshima-u.ac.jp). Translated 2010-Feb-10
		to Javascript by Sean McCullough (banksean@gmail.com). Packaged as a JSM 2017-March-29
		by Eurythrace Perseides.

										--------------------

										Modified BSD License

	Redistribution and use in source and binary forms, with or without modification, are permitted
	provided that the following conditions are met:

	1.	Redistributions of source code must retain the above copyright notices, this list of
		conditions and the following disclaimer.

	2.	Redistributions in binary form must reproduce the above copyright notice, this list of
		conditions and the following disclaimer in the documentation and/or other materials
		provided with the distribution.

	3.	Neither the name of the copyright holder nor the names of its contributors may be used to
		endorse or promote products derived from this software without specific prior written
		permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS	“AS IS” AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR	A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.

										--------------------

	USAGE NOTE:
	Before using, initialize the state by using init_genrand(seed)
	or init_by_array(init_key, key_length).

	Any feedback is very welcome.
	http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
	email: m-mat@math.sci.hiroshima-u.ac.jp

										--------------------

	I've wrapped Makoto Matsumoto and Takuji Nishimura's code in a namespace
	so it's better encapsulated. Now you can have multiple random number generators
	and they won't stomp all over each other's state.

	If you want to use this as a substitute for Math.random(), use the random()
	method like so:

	var m = new MersenneTwister();
	var randomNumber = m.random();

	You can also call the other genrand_{foo}() methods on the instance.
	If you want to use a specific seed in order to get a repeatable random
	sequence, pass an integer into the constructor:
	var m = new MersenneTwister(123);
	and that will always produce the same random sequence.

	Sean McCullough (banksean@gmail.com)

**************************************************************************************************/

var EXPORTED_SYMBOLS = ["MersenneTwister"];

var MersenneTwister = ( function()
{
	var mt = function( seed )
	{
		if( typeof( seed ) === "undefined" ) { seed = new Date().getTime(); }

	//	Period parameters
		this.N = 624;
		this.M = 397;
		this.MATRIX_A = 0x9908B0DF;		// constant vector a
		this.UPPER_MASK = 0x80000000;	// most significant w-r bits
		this.LOWER_MASK = 0x7FFFFFFF;	// least significant r bits

		this.mt = new Array( this.N );	// the array for the state vector
		this.mti = this.N + 1;			// mti == N+1 means mt[N] is not initialized

		this.init_genrand( seed );
	};

//	Initializes mt[N] with a seed.

	mt.prototype.init_genrand = function( s )
	{
		this.mt[0] = s >>> 0;
		for( this.mti = 1; this.mti < this.N; this.mti++ )
		{
			var s = this.mt[this.mti - 1] ^ ( this.mt[this.mti - 1] >>> 30 );
			this.mt[this.mti] = ((((( s & 0xFFFF0000 ) >>> 16 ) * 1812433253 ) << 16 ) + ( s & 0x0000FFFF ) * 1812433253 ) + this.mti;
			/*
				See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier.
				In the previous versions, MSBs of the seed affect
				only MSBs of the array mt[].
				2002/01/09 modified by Makoto Matsumoto.
			*/
			this.mt[this.mti] >>>= 0;	// for machines > 32 bit
		}
	};

//	Generates a random number on the [0,0xFFFFFFFF] integer interval.

	mt.prototype.genrand_int32 = function()
	{
		var y;
		var mag01 = new Array( 0x0, this.MATRIX_A );	// mag01[x] = x * MATRIX_A  for x = 0,1

		if( this.mti >= this.N )						// generate N words at one time
		{
			var kk;

		//	If init_genrand() has not been called, a default initial seed is used.
			if( this.mti === this.N + 1 ) { this.init_genrand( 5489 ); }

			for( kk = 0; kk < this.N - this.M; kk++ )
			{
				y = ( this.mt[kk] & this.UPPER_MASK ) | ( this.mt[kk + 1] & this.LOWER_MASK );
				this.mt[kk] = this.mt[kk + this.M] ^ (y >>> 1) ^ mag01[y & 0x1];
			}
			for( ; kk < this.N - 1; kk++ )
			{
				y = ( this.mt[kk] & this.UPPER_MASK ) | ( this.mt[kk + 1] & this.LOWER_MASK );
				this.mt[kk] = this.mt[kk + ( this.M - this.N )] ^ ( y >>> 1 ) ^ mag01[y & 0x1];
			}
			y = ( this.mt[this.N - 1] & this.UPPER_MASK ) | ( this.mt[0] & this.LOWER_MASK );
			this.mt[this.N - 1] = this.mt[this.M - 1] ^ ( y >>> 1 ) ^ mag01[y & 0x1];

			this.mti = 0;
		}

		y = this.mt[this.mti++];

	//	Tempering
		y ^= ( y >>> 11 );
		y ^= ( y << 7 ) & 0x9D2C5680;
		y ^= ( y << 15 ) & 0xEFC60000;
		y ^= ( y >>> 18 );

		return y >>> 0;
	};

//	Generates a random number on the [0,1) real interval.

	mt.prototype.random = function()
	{
		return this.genrand_int32() * ( 1.0 / 4294967296.0 );	// divided by 2^32
	};

	return mt;
} )();
